% This is the third script Sam Holo's forward model for the effect of
% obliquity on elliptic crater orientations. I briefly note that it is
% actually a function file that is designed to be called many times for
% embarrissingly parallel computations on midway. The first parameter, num,
% is an arbitrary label for which of our generated obliquity pdf's the
% function should be sampling from. The second parameter indicates whether
% we are using the Neukem or Hartmann chronologies. This function loads the
% output from the apply_obliquity script, as well as a database of craters
% (these are actual craters on mars whose latitude-diameter distribution we
% want to match in our predictions) and obliquity histograms. NOTE: the
% crater database and obliquity histograms are NOT relevant to noachian
% crater studies. They apply only to late-Hesperian onward terrains. 

function [] = debiased_pred(num, ishart)

    % Load the requisite data as described above
    load('obliquity_predictions_4.mat','orientationdata','latdata','diamdata');
    load('latehesperian_4test.mat');
    load('obliquity_hists.mat','neuk','hart');
    
    % Store the crater database in a list of "reference" values
    reflats = crater_lats;
    refdiams = crater_diams;
    refages = crater_max_age_bins;
    refors = crater_ors;
    
    % Clear up some memory for good measure
    clear crater_lats crater_diams crater_max_age_bins
    
    % Get the num'th histogram in either the hartmann or neukem system.
    if ishart==1 
        oblhist = hart(num,:,:); %#ok<NODEF> 
    else 
        oblhist = neuk(num,:,:);%#ok<NODEF>
    end
    
    iterations = 1000;
    pred_ors = zeros(iterations, length(refors));
    boot_ors = zeros(iterations, length(refors));
    ksstats = zeros(iterations,1);
    
    for i = 1:iterations
        % Sample the latitude, diameter, and underlying geologic unit
        % age from the "reference" craters
        [templat, index] = datasample(reflats,length(reflats));
        tempdiam = refdiams(index);
        tempage = refages(index);
        tempor = refors(index);

        for j = 1:length(reflats)
            % Randomly sample from the obliquity histograms. The first
            % line in this chunk of code ensures that you consider the
            % histogram of obliquities since the emplacement of the
            % geologic unit on which the reference crater lies. 
            tempoblhist = oblhist(1,:,tempage(j)); 
            temphistcumsum = cumsum(tempoblhist);
            temphistcdf = temphistcumsum./temphistcumsum(end);     
            [~,obl] = min(abs(temphistcdf - rand));

            % Calculate the differences in latitude and diameter between
            % each ensemble model crater and the reference crater
            laterror = abs(latdata(:,obl) - abs(templat(j)));%#ok<NODEF>
            diamerror = abs(diamdata(:,obl) - tempdiam(j))/tempdiam(j); %#ok<NODEF>

            % Restrict the data to the ensemble with the obliquity sampled
            % above (to within 1 degree)
            data = orientationdata(:,obl); %#ok<NODEF>

            % Find the ensemble model craters that are "close" to the
            % reference crater
            restricted_data = data(laterror < 10 & diamerror < .25); %data(laterror < 2.5 & diamerror < .05);

            % Sample from the "close" craters randomly
            pred_ors(i,j) = datasample(restricted_data, 1);
        end
        
        boot_ors(i,:) = tempor;
        [~, ~, teststat] = kstest2(pred_ors(i,:),tempor);
        ksstats(i) = teststat;
    end
    
    % Find the minimum ksstat (this is our "best fit") and save the 
    % data/predictions associated with it
    [~,ind] = min(ksstats);
    best_pred = pred_ors(ind,:);
    best_data = boot_ors(ind,:);
    
    % Save the data. The if statements are just for maintaining conventions
    % in file names. Presumably in any applications of this model, this
    % section will need to be modified.
    if ishart == 1
        newfile = strcat('4hartpred_',num2str(num),'.mat');
    else
        newfile = strcat('4neukpred_',num2str(num),'.mat');
    end
    
    disp(num); 
    save(newfile,'ksstats', 'best_pred','best_data');
end